/*
	===== Credits =====
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+   Dev by Romain Vauchelles	romain.vauchelles@unistra.fr 						+
+   For CNRS 7021               http://www-lbp.u-strasbg.fr/						+
+   1st version    		 1  Avril   2011							+
+   Last version   		 19  Avril   2022							+
													+
Citation :												+
Imaging and Measuring Vesicular Acidification with a Plasma Membrane-Targeted Ratiometric pH Probe	+
Sophie Michelis, Lydia Danglot, Romain Vauchelles, Andrey S. Klymchenko, and Mayeul Collot		+
Analytical Chemistry Article ASAP									+
DOI: 10.1021/acs.analchem.2c00574									+
+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

* Principe
	1) Visualization image
		For each pixel the ratio imageA/imageB is coded by Hue and intensity (imageA + imageB) is coded by brightness ( Lut must have a constant brightness and Saturation ) 

	2) Distribution histogram
		Each bar of the histogram show the sum of all intensities from pixels in the matching range of ratio
		Background pixels (under the user-defined offset) are not considered 
	
* Installation
	Paste RatioloJ_3.7.ijm in C:\Program Files\ImageJ\plugins
	Paste rainbow.lut in	 C:\Program Files\ImageJ\luts

* Imput Images:
	2 tif files with same size and bit depth or one multichannel tif file	

* Output Data: 
	..._Ratio&Intensity.tif			Visualization image : ratio and intensity
	..._Ratio analysis(1).xls		Values of the distribution histogram and macro settings
 	..._Image_parameters(1).txt	   	Parameters of analysis
   Optional output :
 	..._Curve(1).tif				Distribution curve printed as an image
 	..._selections.zip				user defined ROI
 	..._Intensity.tif				Intensity image : the offset is applied on the two raw images (background pixels are decreased to 0),
									 then they are summed and finally the optional smoothing is applied ( Gaussian filter of 0.5 pixels)
 	..._Ratio.tif					Ratio image: the row images are divided and all the NaN and infinite pixels are turned to 0


* Version notes:
	2.5		- addition of the distribution curve									[ 25/01/2013 ]
	2.6		- bug with some curve scales			 							[ 12/10/2016 ]
	2.7		- suppression of the peak in the first value for histograms starting at 0				[ 21/10/2016 ]
	2.8		- optional ROI selections										[ 13/06/2018 ]
			- print the mean value on the histogram	
	2.9		- option restart macro is available at the end of analysis						[ 18/07/2018 ]

	3.0a	- Global review of the full script										[ 14/08/2018 ]
			- binning option is desactivated (now in comment)
			- antialiasing filter is now operational (smooth option)
			- antialiasing filter is switchable between "Smooth" or "gaussian"
	3.0b	- first user interface is reviewed, the macro is now compatible with simple of multichannel images 
			- debug of restart fonction
	3.0c	- the creation of the distribution curve is now a macro function
			- graphs on raw data (first analyses) and dialog boxes are now Nonblocking windows
			- All ROI are created at once
 	3.0d	- A mask of the back ground pixels(offset) is used to suppress areas created by antialiasing filters
			- artifact with ratio image filtering is fixed
			- new output parameter files
			- temps files are deleted
			- Ratio distribution analysis is now optional
			- Suppression of binning option  
	3.beta	- Following option are now in comments
			- binning
			- presmooth
			- intensity smooth
		   -> this version is distributed for final user test
		   
	3.0		- Compatibility windows and Macintosh									[ 9/11/2020 ]
			- Ratio scale-bar is untie for ratioloJ image in option
	3.1		- Saving the output files Intensity, Ratio and histograms is now optional
			- Definitive suppression of binning and presmooth options  
			- Mean ratio without intensity ponderation is now added in the result table
	3.2		- Batchmode optimization 										[ 04/05/2021 ]
			- Optimizations of script and interface
	3.3		- Bug correction for file with space in the name (sicnce last imageJ update)				[ 31/05/2021 ]
	3.4		- RatioloJ image keep the scale of input imageA  (pixel size in tif metadata) 
	3.4.1	- dialogbox positions
			- new output parameter files
			- Plot of all curves appears in one graph if there are severals ROI
	3.4.2	- All histogram values are saved in a unique tab file
			- *.tsv files replace excel files
	3.5		- New project name !! -- RatioloJ --  									[ 28/09/2021 ]
			- .tsv files are not in the distribution curve anymore						
			- Bug correction in .tsv file if there is only one selection
			- Bug correction for meanRat in output file
			- New output file names
			- Color wheel for display multiple curves
	3.6		- Ratio image is displayed with the brightness/contrast matching with ratioloJ image			[ 10/03/2022 ]
			- final dialog box is nonblocking window									[ 15/03/2022 ]
	3.7		- suppression of French special characters (ANSI coding for fixing FIJI bug)		[ 08/04/2022 ]
	3.8		- bug if source image is not a square X and Y were inverted in the image scanning to make the output histogram [19/04/2022]
			- bug in restart function if going from multiple selection tu full image analysis
	
* Potential developments
	+ Save and load settings in a log file
	+ convert the macro in a java plugin for imageJ or Icy
	+ Extend analyses to 3D or timelapse stacks or a directory batch processing
*/

//++++++++++ Default parameters ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	defChanA=0;
	defChanB=1;

   DefaultBack1 = 0;		 // Background level
   DefaultBack2 = 0;		 // pixel below this value will be down to 0

   preSmouthRatio = true;	 // Anti-aliasing filter: it's a Gaussian filter of 0.5 pixels applied on the ratio image (not on the intensity image) 

   IntSatNorm   = 0.8;		 // Saturation of intensity picture (between 0 and 1)
   defaultgamma = 1;		 // Gamma, gamma=1: image is not changed, gamma<1: raise lower levels, gamma>1: raise upper levels

   spectroSat   = 0.95;		 // Saturation of ratio picture (between 0 and 1)
   spectroSeuil = 0.5;		 // Threshold of ratio picture (between 0 and 1)
   ImposeDefautRatio = true;	 // if ImposeDefautRatio =false the suggested values matching with the two values above (spectroSat and spectroSeuil)
				 // if ImposeDefautRatio =true  the suggested values are the two values below (spectroTop and spectroBot)
   spectroTop = 0.1;		 // Saturation of ratio picture
   spectroBot =   0;		 // Threshold of ratio picture
   
   defaultlut = "Rainbow";	 // "Rainbow" is highly recommended
   FontSize   = 30;	 	 // Font size of the scale bar annotations

   DistributionAnalyse = true; 	 // Activation of the distribution analyses (graph and table)
   scaleDetached = false;	 // If true, the bar appears in a different image
   scaleTag = false;		 // If scaleTag=true, no tag are shown in the ratio scale (ratio and intensity image)
				 // Suggested limits of the X axis on the curve are the same of the ratio scaling on ratio&intensity image (spectroTop and spectroBot)
   defaultStep = 0.005;		 // Step of the histogram scaling (binning), it must be the same for the different conditions to compare (because of aliasing)
 
   keepRaw   = false;		 // Save raw ratio and raw intensity images these are temp files, it's recommended to delete them to save hard drive space 
   keepIjHis = false;		 // Save imageJ histogram, if not saved, histograms values are in the output tab file.

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

 // **** Preparation ImageJ
	MacroNa="RatioloJ";
	Vers=""+"3.8";
	requires("1.53q");
	ScreenClean();
	run("Set Measurements..."," min redirect=None decimal=3");
	createLutList();
	setBatchMode(true);
// *** Start variables
	roiCount  = -1;
	multiPlot = false;
	ratMoy=newArray(1);

 // =====================
 // **** Images Source

	  NativePathA = File.openDialog("Open numerator image");
	        NameA = File.getName(NativePathA);
	          Dir = File.directory;
	 open(NativePathA);
	  getDimensions(Large,Haut,channelsA,slicesA,framesA);
	  getPixelSize(unit, pixelWidth, pixelHeight);

	  if (channelsA<=defChanA) defChanA=0;
	    AChanList=newArray(channelsA);
	  for (i=0; i<channelsA; i++)
	    AChanList[i]="Channel "+i+1;

	if (channelsA==1)
	 {
	     NativePathB = File.openDialog("Open second image");
	   open(NativePathB);
	           NameB = getTitle();
	   getDimensions(widthB,heightB,channelsB,slicesB,framesB);
	   if (channelsB<=defChanB) defChanB=0;
	    BChanList=newArray(channelsB);
	  for (i=0; i<channelsB; i++)
	    BChanList[i]="Channel "+i+1;
	   close(NameB);
	 }
	else
	 {
	     NativePathB = NativePathA;
	           NameB = NameA;
	       channelsB = channelsA;
	    if (channelsB<defChanB) defChanB=0;
	    BChanList=newArray(channelsA+1);
	  for (i=0; i<channelsA; i++)
	    BChanList[i]="Channel "+i+1;
	    BChanList[i]="Select another file";
	 }

	    DialogContinue=true;
	while(DialogContinue)
	 {
		Dialog.create("Operation choice");
		 Dialog.addMessage("Select image A");
		  Dialog.addChoice(NameA, AChanList, AChanList[defChanA]);
 	 	 Dialog.addMessage("Select image B");
	 	  Dialog.addChoice(NameB, BChanList, BChanList[defChanB]);
		  Dialog.addRadioButtonGroup("Operation choice", newArray("A/B","B/A"), 1, 2,"A/B");
	 	Dialog.show();

	  ACChoice = substring(Dialog.getChoice(), 8, 9);
	  BCChoice = substring(Dialog.getChoice(), 8, 9);
	 DivChoice = Dialog.getRadioButton;

	    Numer =substring(DivChoice, 0, 1);
	    Denom =substring(DivChoice, 2, 3);

		if(BCChoice=="n")
	 	  {
		   NativePathB=File.openDialog("Ouvrez la seconde image");
		   NameB=File.getName(NativePathB);
		  open(NativePathB);
		   getDimensions(widthB,heightB,channelsB,slicesB,framesB);
		   if (channelsB<=defChanB) defChanB=0;
		    BChanList=newArray(channelsB);
		  for (i=0; i<channelsB; i++)
		    BChanList[i]="Channel "+i+1;
		  close(NameB);
	 	  }
		else
	 	  {
		   open(NativePathB);
	 	   DialogContinue=false;
	 	  }
 	 }
 
	if (Numer=="A")
	 {
	  NativePathNum = NativePathA;
	  NameNum	= NameA;	
	  ChanNum	= ACChoice;
	  	if (channelsA==1)
	  	 MultiChanNum=false;
	  	else
	  	 MultiChanNum=true;
	  NativePathDen	= NativePathB;
	  NameDen	= NameB;	
	  ChanDen	= BCChoice;
	  	if (channelsB==1)
	  	 MultiChanDen=false;
	  	else
	  	 MultiChanDen=true;
	 }
	else
	 {
	  NativePathNum	= NativePathB;
	  NameNum	= NameB;	
	  ChanNum	= BCChoice;
	  	if (channelsB==1)
	  	 MultiChanNum=false;
	  	else
	  	 MultiChanNum=true;
	  NativePathDen	= NativePathA;
	  NameDen	= NameA;	
	  ChanDen	= ACChoice;
	  	if (channelsA==1)
	  	 MultiChanDen=false;
	  	else
	  	 MultiChanDen=true;
	 }

	Prefix=substring(NameA,0,lengthOf(NameA)-4);

	   selectWindow(NameNum);
	    NameNum=NameNum+"N"; 
	    rename(NameNum);	// run("32-bit");
	   selectWindow(NameDen);
	    NameDen=NameDen+"D";
	    rename(NameDen);	// run("32-bit");

 //*** Repertoire de sortie 

	 OutPath=Dir+File.separator+Prefix+"_Ratio";
	  texist=1; 
          Dupli=File.exists(OutPath);
	  while (Dupli==1){texist++;OutPath=Dir+File.separator+Prefix+"_Ratio ("+texist+")";Dupli=File.exists(OutPath);} 
	  File.makeDirectory(OutPath);

 // ###########################
 // **** Boucle principale **** 

	if (MultiChanNum)
	 {
	   selectWindow(NameNum);
	    Stack.setChannel(ChanNum);
	    run("Reduce Dimensionality...", " ");
	 }

	if (MultiChanDen)
	 {
	   selectWindow(NameDen);
	    Stack.setChannel(ChanDen);
	    run("Reduce Dimensionality...", " ");
	 }

	 Continue=true;
	 CompteurBoucle=0;
	while (Continue==true)
	{
	 CompteurBoucle=CompteurBoucle+1;
	setFont("SansSerif", 30," antialiased");

// ============================================
// |---- Chapitre 1 mesures preliminaires ----|
// ============================================

 if(CompteurBoucle==1)
  {
 //****** Creation de Raw Intensity (Somme des deux canaux) et Raw Ratio (Valeurs reelles)
	imageCalculator("Divide create 32-bit",NameNum,NameDen);
	 getRawStatistics(a,b,RatMin,RatMax);		    // RatMax=Ratio Max mesuree
	// RatSat=floor(RatMax*IntMax);			   // IntSat=saturation que l'on va proposer IntSatNorm=entre 0 et 1, defini dans les parametres

	run("Rainbow");
	   InfinityToNAN();
	   SetNAN(0);
	saveAs("Tiff",OutPath+File.separator+Prefix+"_Ratio");
	   rename("Ratio");

	imageCalculator("Add create 32-bit",NameNum,NameDen);
	  getRawStatistics(a,b,MinSom,IntMax);		 // IntMax=intensite max mesuree
	  IntSat=floor( IntMax*IntSatNorm);		// IntSat=saturation que l'on va proposer IntSatNorm=entre 0 et 1, defini dans les parametres
	 rename("Rawintensity");

          run("Histogram", "bins=256 x_min=0 x_max="+IntMax+" y_max=Auto");
	  rename("Histogram Rawintensity");

	setBatchMode("exit and display");
	run("Tile");
	   HistoMin=RatMin; HistoMax=RatMax; HistoStep=defaultStep;
           WholIma=true; 
	ratMoy[0]=RatioDistributionCurve("Rawintensity","Ratio","Raw Image",false,true);
  }	
 else
  {
	  open(NativePathNum);rename(NameNum);
	      getDimensions(widthNum,heightNum,channelsNum,slicesNum,framesNum);	
	    if (channelsNum>1)
	     {
		Stack.setChannel(ChanNum);
		run("Reduce Dimensionality...", " ");
	     }
	  open(NativePathDen);rename(NameDen);	
	      getDimensions(widthDen,heightDen,channelsDen,slicesDen,framesDen);	
	    if (channelsDen>1)
	     {
		Stack.setChannel(ChanDen);
		run("Reduce Dimensionality...", " ");
	     }
	setBatchMode("exit and display");
  }

 //----------------------
 //**** INTERFACE II **** 
 //----------------------
 //****** Interface utilisateur

	   selectWindow(NameNum); run("Out [-]");
	   selectWindow(NameDen); run("Out [-]");
	run("Tile");
	  if (ImposeDefautRatio==false)
	   {
	     spectroTop=RatMax*spectroSat;
	     spectroBot=RatMin*spectroSeuil;
	   }
	 Dialog.createNonBlocking("Visualisation And Analysis Parameters");
	     Dialog.addMessage("----- Background level (offset) -------------------------------");
		Dialog.addNumber(substring(NameNum,0,NameNum.length-1)+"  C-"+ACChoice,DefaultBack1);
		Dialog.addNumber(substring(NameDen,0,NameDen.length-1)+"  C-"+BCChoice,DefaultBack2);
	     Dialog.addMessage("----- Intensity (brightness) ------------------------------------");
		Dialog.addMessage ("Maximum values mesured: "+IntMax);
		Dialog.addNumber( "Saturation",       IntSat);
		Dialog.addNumber( "Gamma correction", defaultgamma);
	     Dialog.addMessage("----- Ratio (color) ------------------------------------------------");
		Dialog.addMessage ("Mesured values are between "+RatMin+" and "+RatMax);
		Dialog.addNumber( "Minimum scale", spectroBot);
		Dialog.addNumber( "Maximum scale", spectroTop);
		Dialog.addCheckbox("Smooth (Recommanded for distribution analysis)", preSmouthRatio);
	     Dialog.addMessage("----- Display options --------------------------------------------");
		Dialog.addChoice("Lookup Table",LutList,defaultlut);
		Dialog.addCheckbox( "Detach ratio scale bar" , scaleDetached);
		Dialog.addCheckbox( "Hide ratio scale tags" ,  scaleTag);
	     Dialog.addMessage("----- Ratio Distribution analysis -----------------------------");
		Dialog.addCheckbox("Perform distribution analysis", DistributionAnalyse);
		Dialog.addNumber( "Histogram minimum" , spectroBot);
		Dialog.addNumber( "Histogram maximum" , spectroTop);
		Dialog.addNumber( "Histogram step" ,    defaultStep);
	     Dialog.addMessage("----- Save options -----------------------------------------------");
		Dialog.addCheckbox( "Save raw ratio and intensity images", keepRaw);
		Dialog.addCheckbox( "Save imageJ histogram",             keepIjHis);
	 Dialog.show();

				BackNum	= Dialog.getNumber();		DefaultBack1 = BackNum;   // memorisation pour le prochain tour
				BackDen	= Dialog.getNumber();		DefaultBack2 = BackDen;

				 IntSat	= Dialog.getNumber();
				 Igamma	= Dialog.getNumber();
			   defaultgamma	= Igamma;

			     spectroBot	= Dialog.getNumber();
			     spectroTop	= Dialog.getNumber();
			      lutfinale	= Dialog.getChoice();

				RSmooth	= Dialog.getCheckbox();
			 preSmouthRatio	= RSmooth;

			     defaultlut = lutfinale;
			  scaleDetached = Dialog.getCheckbox();
			      scaleTag	= Dialog.getCheckbox();

		    DistributionAnalyse	= Dialog.getCheckbox();
			       HistoMin = Dialog.getNumber();
			       HistoMax	= Dialog.getNumber();
			      HistoStep	= Dialog.getNumber();
			    defaultStep	= HistoStep;

			  	keepRaw = Dialog.getCheckbox();
			      keepIjHis	= Dialog.getCheckbox();

	 if(CompteurBoucle==1)
	  {
		close("Rawintensity");
	    if(isOpen("Histogram Rawintensity"))
		close("Histogram Rawintensity");
	    if(isOpen("Ratio distribution RoiRaw Image"))
		close("Ratio distribution RoiRaw Image");
	  }
	 else 
	  {
		open(OutPath+File.separator+Prefix+"_Ratio.tif");
		rename("Ratio");
 	  }

// =============================================
// |---- Chapitre 2 Outil de visualisation ----|
// =============================================

	setBatchMode(true);

 //--------------------------
 //**** IMAGE INTENSITE  **** 
 //--------------------------

 //****** Nouvelle image d'intensite (somme des canaux avec les offset) 
	selectImage(NameNum);
	   DownLowNul(BackNum);
	selectImage(NameDen);
	   DownLowNul(BackDen);
	imageCalculator("Add create 32-bit",NameNum,NameDen);
	   run("Grays");
	saveAs("Tiff",OutPath+File.separator+Prefix+"_Intensity");
        close(NameNum);
        close(NameDen);

 //****** Masque des offset
	selectImage(Prefix+"_Intensity.tif");
	   run("Duplicate...", "title=mask");
	     setThreshold(0,0);
	     run("Convert to Mask");
	     run("Invert");
	     run("Divide...", "value=255");
	   saveAs("Tiff",OutPath+File.separator+Prefix+"_OffsetMask");

	 selectWindow(Prefix+"_Intensity.tif");
	run("Duplicate...", "title=Intensitemp");
           run("Max...", "value="+IntSat);		      // Saturation de l'image	
	   run("Divide...", "value="+IntSat);		     // Normalisation entre 0 et 1;
          if (Igamma!=1) run("Gamma...", "value="+Igamma);  // Correction gamma

	imageCalculator("Multiply create 32-bit", "Intensitemp",Prefix+"_OffsetMask.tif");
	     rename("Intensity");
	     close("Intensitemp");

 //** echelle (elargissement)
	run("Canvas Size...", "width="+Large+60+" height="+Haut+" position=Center-Left zero");
	  makeRectangle(Large,0,Large+60,Haut);
	  run("Set...", "value=1");
	  run("Select None");
	  setMinAndMax(0,1);

 //---------------------
 //**** IMAGE RATIO ****
 //---------------------

        selectWindow("Ratio");
	 if (RSmooth==true) AAFilter();

 //****** Echelle (Valeurs etalees entre RatMin et le RatMax)
	newImage("Rampe", "32-bit Ramp", Haut, Large+60 , 1);
	 run("Rotate 90 Degrees Left");
	 run("Multiply...", "value="+(spectroTop-spectroBot));
	 run("Add...", "value="+spectroBot);
	 setMinAndMax(spectroBot,spectroTop);

 //****** Copie et deplacement de l'image ratio dans l'image rampe
	selectWindow("Ratio");
         makeRectangle(0,0,Large,Haut);
	 run("Copy");
	selectWindow("Rampe");
	 run("Paste");
	 makeRectangle(0,0,Large+30,Haut);
	 run("Translate...", "x=-30 y=0 interpolation=None");
	 run("Select None");

 //***** Convertion en image couleur
	 selectWindow("Rampe");
	   run(lutfinale);
	   setMinAndMax(spectroBot,spectroTop);
	   run("RGB Color");
           rename("Ratio & intensity");

 //----------------------------
 //**** RATIO & INTENSITE *****
 //----------------------------

   //** Multiplication par l'image d'intensite	: les 3 couleurs primaires sont traitees independament puis recombinees
	 run("Split Channels");
	   imageCalculator("Multiply create 32-bit","Ratio & intensity (red)","Intensity");
	   imageCalculator("Multiply create 32-bit","Ratio & intensity (green)","Intensity");
	   imageCalculator("Multiply create 32-bit","Ratio & intensity (blue)","Intensity");

	 run("Merge Channels...", "red=[Result of Ratio & intensity (red)] green=[Result of Ratio & intensity (green)] blue=[Result of Ratio & intensity (blue)] gray=*None* create keep");
	  rename("Ratio&Intensity");
	   Stack.setChannel(1); setMinAndMax(0, 255);		//Les displays des 3 canaux sont normalises pour conserver la teinte
	   Stack.setChannel(2); setMinAndMax(0, 255);
	   Stack.setChannel(3); setMinAndMax(0, 255);
	 run("RGB Color");
	 setVoxelSize(pixelWidth, pixelHeight, 1, unit);

 //****** Annotations de la barre d'echelle
	if (scaleTag==false)
	 {
		setFont("SansSerif", FontSize," antialiased");
	    scale = (spectroTop-spectroBot)/4;
		 run("Colors...", "foreground=black background=white selection=yellow");
		  drawString(spectroBot,Large+1,Haut-1);
	          drawString(spectroBot+scale*3,Large+1,Haut/4+4);
		  drawString(spectroBot+scale*2,Large+1,Haut/2+4);
		  drawString(spectroBot+scale,Large+1,Haut*3/4+4);
		  drawString(spectroTop,Large+1,38);

		  makeRectangle(Large,0,40,4);		run("Fill", "slice");
		  makeRectangle(Large,Haut/4+2,40,4);	run("Fill", "slice");
		  makeRectangle(Large,Haut/2+2,40,4);	run("Fill", "slice");
		  makeRectangle(Large,Haut*3/4+2,40,4);	run("Fill", "slice");
		  makeRectangle(Large,Haut-4,40,4);	run("Fill", "slice");
		 run("Colors...", "foreground=whiteblack background=black selection=yellow");
		run("Select None");
	 }
	    saveAs("Tiff",OutPath+File.separator+Prefix+"_Ratio&Intensity("+texist+")");

 //****** Show 
	    setBatchMode(false);
	      while(nImages>0) close();
	       open(OutPath+File.separator+Prefix+"_Ratio.tif");			run("Set... ", "zoom=75");    setMinAndMax(spectroBot,spectroTop);
		rename("Ratio");
	       open(OutPath+File.separator+Prefix+"_Intensity.tif");	  		run("Set... ", "zoom=75");
	       open(OutPath+File.separator+Prefix+"_Ratio&Intensity("+texist+").tif");	run("Set... ", "zoom=75");
	    run("Tile");

// =======================================================
// |---- Chapitre 3 Mesures de distribution du ratio ----|
// =======================================================

    if (DistributionAnalyse)
	{
// ****** options de smoothing

	 if (RSmooth==true)
	  {
	    setBatchMode(true);
	   selectWindow("Ratio"); 		zoomR=getZoom()*100;
	    AAFilter();
   	   open(OutPath+File.separator+Prefix+"_OffsetMask.tif");
	  imageCalculator("Multiply create 32-bit","Ratio",Prefix+"_OffsetMask.tif");
	    close("Ratio");
	    close(Prefix+"_OffsetMask.tif");
	    setBatchMode("exit and display");
	   selectWindow("Result of Ratio"); 
	    rename("Ratio");	run("Set... ", "zoom="+zoomR); setMinAndMax(spectroBot,spectroTop);
	  }

// ****** Zone de mesures

	  run("ROI Manager...");
	  run("Tile");
	 selectWindow(Prefix+"_Ratio&Intensity("+texist+").tif");

		Dialog.createNonBlocking("NonBlockDial");
		       Dialog.setLocation( DialogPositionX (Prefix+"_Ratio&Intensity("+texist+").tif",280), DialogPositionY ( Prefix+"_Ratio&Intensity("+texist+").tif" ) );
		       Dialog.addMessage("Add regions to analyse in the RoiManager\nThe whole image will be analysed if empty");
		Dialog.show();
	
			roiManager("Show None");
			
	roiCount=roiManager("Count");
	   WholIma=false;
	  if(roiCount==0)
	   {
	    selectWindow(Prefix+"_Intensity.tif");
	    run("Select All");
	    roiManager("Add");
	    roiManager("Select",0);
	    roiManager("Rename","Whole image");
	    roiCount=1;
	    WholIma=true;
	   }
	  else
	    for(iRoi=0; iRoi<roiCount; iRoi++)
	     {
	      roiManager("Select", iRoi);
	      roiManager("Rename", "Roi-"+iRoi+1);
	      roiManager("save", OutPath+File.separator+Prefix+"_selections ("+texist+").zip"); 
	     }
	     roiManager("deselect"); 

	  if(roiCount>1)
	     {
		Plot.create ("All distributions","Ratio","Cumulated Intensities");
			Plot.setLimits (HistoMin,HistoMax,0,1);
	    		Plot.setColor ("Blue");
        		Plot.show();
			Plot.freeze(true);	
		multiPlot=true;
	     }
	
	 for(iRoi=0; iRoi<roiCount; iRoi++)
	  {
           setBatchMode(true);
		selectWindow("Ratio"); 	run("Select None");
		   run("Duplicate...", "title="+Prefix+"_Ratio_Roi"+iRoi);
		    rename(Prefix+"_Ratio_Roi"+iRoi);  				// bug si le nom contient des espaces
		    roiManager("Select", iRoi); run("Make Inverse");
		    if(selectionType!=-1)
		    run("Set...", "value=0");
		    run("Select None");

		selectWindow(Prefix+"_Intensity.tif"); run("Select None");
		   run("Duplicate...", "title="+Prefix+"_Intensity_Roi"+iRoi);
		    rename(Prefix+"_Intensity_Roi"+iRoi);  			// bug si le nom contient des espaces
		    roiManager("Select", iRoi); run("Make Inverse");
		    if(selectionType!=-1)
		    run("Set...", "value=0");
		    run("Select None");
			
	    ratMoy[iRoi]=RatioDistributionCurve(Prefix+"_Intensity_Roi"+iRoi,Prefix+"_Ratio_Roi"+iRoi,iRoi+1,true,WholIma);
	        close(Prefix+"_Ratio_Roi"+iRoi);
	        close(Prefix+"_Intensity_Roi"+iRoi);
	  }
	}
	  run("Select None");

// ****** Proposition de changer les valeurs entrees au debut

	setBatchMode("exit and display");
	  run("Tile");
	Dialog.createNonBlocking("Change");
	 Dialog.addMessage("Do you want to run the macro another time\n and change the values ?")
	 Dialog.addRadioButtonGroup("",newArray("Yes","No"),1,2,"No");
	Dialog.show();
	TryAgain=Dialog.getRadioButton();

	if (TryAgain=="Yes")
		{
		   Continue=true;
		   while(nImages>0) close ();
		   run("Clear Results");
		   run("Set Measurements..."," min redirect=None decimal=3");
		   ScreenClean();
		   	roiCount  = -1;
			multiPlot = false;
			ratMoy=newArray(1);   
		}
	else
		{
		     Continue=false;
		      File.delete(OutPath+File.separator+Prefix+"_OffsetMask.tif");
		    if(keepRaw==false)
		     {
		      File.delete(OutPath+File.separator+Prefix+"_Ratio.tif");
		      File.delete(OutPath+File.separator+Prefix+"_Intensity.tif");
		     }
		     selectWindow("Log"); run ("Close"); wait(10);
			 
			if(scaleDetached==true)
			 {
			   selectWindow(Prefix+"_Ratio&Intensity("+texist+").tif"); 
			   makeRectangle(0,0,Large,Haut);
			   run("Make Inverse");
			   run("Duplicate...", "title=Bar");
			   run("Out [-]");
		      saveAs("Tiff",OutPath+File.separator+Prefix+"_Ratio Scalebar("+texist+")");		  
			 selectWindow(Prefix+"_Ratio&Intensity("+texist+").tif");
			   makeRectangle(0,0,Large,Haut);
			   run("Crop");
			  saveAs("Tiff",OutPath+File.separator+Prefix+"_Ratio&Intensity("+texist+")");
			 }

	  	      if(WholIma)		//et donc roiCount>0
			close("ROI Manager");
	 	run("Tile");
	     selectWindow(Prefix+"_Ratio&Intensity("+texist+").tif");

	  	      if(roiCount==1)
		       {roiManager("Show All without labels");roiManager("Deselect"); run("Select None");}
		      if(roiCount>1)
		       {roiManager("Show All with labels");   roiManager("Deselect"); run("Select None");}
		}
	}
// ===========================================================
// |---- Sauvegarde des parametres et donnes d'analyses ----|
// ===========================================================
 
//****** Sauvegardes des parametres
 
	    date=horodate();
	txt=File.open(OutPath+File.separator+Prefix+"_Image_parameters("+texist+").txt");
	  print(txt,"============ "+MacroNa+" "+Vers+" =================== ");
		print(txt,"ImageJ\t"+getVersion());
		print(txt,"Date\t"+date);
		print(txt,"\nA:\t"+substring(NameNum,0,NameNum.length-1)+" C-"+ ChanNum);
		print(txt," was divided by ");
		print(txt,"B:\t"+substring(NameDen,0,NameDen.length-1)+" C-"+ ChanDen);
		print(txt," ");
          print(txt,"============ Preprocessing ================== ");
		print(txt," ");
		print(txt,"Offset (Background depletion) :");
		print(txt,"    A:\t"+BackNum);
		print(txt,"    B:\t"+BackDen);
		print(txt," ");
          print(txt,"============ Intensity A+B (brightness) ===== ");
		print(txt," ");
		print(txt,"Maximal Intensity mesured:   "+IntMax);
		print(txt,"Maximal Intensity displayed (saturation):  "+IntSat);
	     if (Igamma==1) 	print(txt,"No gamma correction");
	      else 		print(txt,"Gamma correction: "+Igamma);
		print(txt," ");
	  print(txt,"============ Ratio A/B (color) ============== ");
		print(txt," ");
	     if (RSmooth==true) 	print(txt,"Ratio image was filtered by a gaussian filter 0.5 pixels ");
	      else 		print(txt,"No smoothing was performed on ratiometric image");
		print(txt," ");
		print(txt,"Minimal ratio shown:\t"	+spectroBot);
		print(txt,"Maximal ratio shown:\t"	+spectroTop);
		print(txt,"\nMesures on whole image:");
		print(txt,"Minimal ratio mesured:\t"	+RatMin);
		print(txt,"Maximal ratio mesured:\t"	+RatMax);
		print(txt," ");
	  print(txt,"============ Distribution Histogram ========= ");
		print(txt," ");

		if (!DistributionAnalyse)
			print(txt,"No histogram created");
		else
		 {
		    print(txt,"Histogram minimum\t"+ spectroBot);
		    print(txt,"Histogram maximum\t"+ spectroTop);
		    print(txt,"Histogram step   \t"+ defaultStep);
		    print(txt,"");
		  if ( WholIma )
		    print(txt,"Distribution histogram was performed on whole image");
		  else if (roiCount==1)
		    print(txt,"Distribution histogram was performed on 1 user defined region of interest ");
		  else 
		    print(txt,"Distribution histograms were performed on "+roiCount+ " user defined region of interest ");
		 }
	File.close(txt);

// **** Exportation des histogrammes de distribution (extraction des courbes IJ et sauvegarde en .tsv)

	if(DistributionAnalyse)
	 {
	      if(multiPlot)
		selectWindow("All distributions");
		      else
		selectWindow(Prefix+"("+texist+")_Distribution Roi 1.tif");
  
	  	Plot.showValues();

	 	 DistLength = nResults;
	 	   ColRatio = newArray(DistLength);
	 	   CurveOne = newArray(DistLength);

		      if(multiPlot)
			for (i=0;i<DistLength;i++)
			 {
			   ColRatio[i] = getResult("X0", i);
			   CurveOne[i] = getResult("Y0", i);
			 }
		      else
			for (i=0;i<DistLength;i++)
			 {
			   ColRatio[i] = getResult("X", i);
			   CurveOne[i] = getResult("Y", i);
			 }

	   	 IJ.renameResults("Results","Source");
				setResult("Ratio",0, "MeanValues" );
				setResult("Roi1", 0,  ratMoy[0]   );
			for (i=0;i<DistLength;i++)
			 {
				setResult("Ratio",i+1,  ColRatio[i] );
				setResult("Roi1" ,i+1,  CurveOne[i] );
			 }
		IJ.renameResults("Results","Dest"); IJ.renameResults("Source","Results");

		if (roiCount>1)
		 {
		  SupCurves = newArray(nResults);
	
			for (i=2;i<=roiCount;i++)
			 {
			   for (j=1;j<DistLength;j++)
			    {
				SupCurves[j] = getResult("Y"+i-1, j);
			    }
			IJ.renameResults("Results","Source");IJ.renameResults("Dest","Results");
				setResult("Roi"+i, 0, ratMoy[i-1]);
			   for (k=0;k<DistLength;k++)
			    {
		    		setResult("Roi"+i,  k+1,  SupCurves[k]);
			    }
			IJ.renameResults("Results","Dest"); IJ.renameResults("Source","Results");
			 }
	     	     if(keepIjHis)
		      {
			selectWindow("All distributions");
			saveAs("Tiff",OutPath+File.separator+Prefix+"("+texist+")_All distributions.tif");
		      }
		     }

		  selectWindow("Results"); run("Close");
		  IJ.renameResults("Dest","Results");
		saveAs("Results", OutPath+File.separator+Prefix+"("+texist+")_Distributions.tsv");
		  selectWindow("Results"); run("Close"); 
	 }
 exit();

// ###########################

   //Script end\\ 
  //000000000000\\

// ===================
// |----FONCTIONS----|
// ===================

// ================================
// ==== courbe de distribution ====

 function RatioDistributionCurve(RdcIntens,RdcRatio,RdcRoiName,RdcSave,RdcWholIma)
  {
	setBatchMode(true);  
	HistoSize  = floor((HistoMax-HistoMin)/HistoStep)+1;
	RatDistrib = newArray(HistoSize);
	for(i=0;i<Large;i++)
	  {
	     for(j=0;j<Haut;j++)
		{
		  selectImage(RdcRatio);
		   raton = getPixel(i,j);
		    if (raton<HistoMax && raton>=HistoMin)
			{
				selectImage(RdcIntens);
				RatDistrib[floor((raton-HistoMin)/HistoStep)] += getPixel(i,j);
		    }
		}
	  }
   	   RatDistrib[0]=0;  //suppression du pic artefactuel a la colonne  0

 // *** Calcul du ratio moyen
	  SommeIntensite=0;
	  SommeIntensiteRatio=0;
	for (i=0;i<RatDistrib.length;i++)
	 {
	  SommeIntensiteRatio += RatDistrib[i]*(HistoMin+i*HistoStep);
	  SommeIntensite += RatDistrib[i];
	 }
	MeanRat = SommeIntensiteRatio/SommeIntensite;

	if (SommeIntensiteRatio==0)
	   MeanRat ="No values found in this histogram range";
 
// ****** Affichage de la courbe

	setBatchMode(false);  
			Array.getStatistics(RatDistrib,RDmin,RDmax,RDmean,RDdev);
			AbscisAx=newArray(HistoSize);
		  for(i=0;i<HistoSize;i++)   AbscisAx[i]=HistoMin+i*HistoStep;

			Plot.create (Prefix+"("+texist+")_Distribution Roi "+RdcRoiName+".tif","Ratio","Cumulated Intensities",AbscisAx,RatDistrib);
			  Plot.setLimits (HistoMin,HistoMax,0,RDmax*1.05);
			  Plot.setColor ("Black");
				Plot.show();
				setColor(0, 0, 0);
		  	  setFont("SansSerif", 15);
	          	  setForegroundColor(0,0,0);
			if (SommeIntensiteRatio==0)
		 	  drawString(MeanRat, 80, 60);
			else
		 	  drawString("Mean Ratio = "+d2s(MeanRat,3), 80, 60);
			Plot.freeze(true);

	    if(RdcSave)
	      if(keepIjHis)
		saveAs("Tiff",OutPath+File.separator+Prefix+"("+texist+")_Distribution Roi "+RdcRoiName+".tif");

// ****** Courbe multiple
	       if(multiPlot)
		{
		  selectWindow("All distributions");
			Plot.freeze(false);
			Plot.add("line",AbscisAx,RatDistrib);
	    		Plot.setColor (colorWheel(RdcRoiName));
			Plot.setLimitsToFit();	
			Plot.freeze(true);
		}
    return MeanRat
  }

// ==============================
// ==== Anti Aliasing Filter  ====

	 function AAFilter()
	  {
	    run("Gaussian Blur...", "sigma=0.5");
	  }

// =======================
// ==== Liste des lut ====

	   var LutList;
	 function createLutList()
	  {
		 LutDir = getDirectory("luts");
		LutList = getFileList(LutDir);

	       err="No LUTs in the '/ImageJ/luts' folder";
		if (!File.exists(LutDir))	exit(err);
		if (LutList.length==0)		exit(err);

		for (i=LutList.length-1; i>=0; i--)
                	if (endsWith(LutList[i],".lut")==false)
				LutList= Array.deleteIndex(LutList, i);
			else
				LutList[i]=substring(LutList[i],0,lengthOf(LutList[i])-4);
	  }

// =========================
// ==== infinity to NAN ====

	function InfinityToNAN()
	 {
		getRawStatistics(INa,INb,INc,INMax);
		 run("Max...", "value="+INMax+2);
		setAutoThreshold("Default dark");
		setThreshold(0,INMax+1);
		 run("NaN Background");
	 }

// =================================
// ==== Set value of NAN pixels ====

	function SetNAN(valNAN)
	 {
	   getDimensions(SNVwidth,SNVheight,SNVc,SNVd,SNVe);
	   for (i=0;i<SNVwidth;i++)
	    {
	     for (j=0;j<SNVheight;j++)
	      {
		if( isNaN( getPixel(i,j) ) )
		 setPixel(i,j,valNAN);
	      }
		}
	 }

// =================================
// ==== Down lowers pixels to 0 ====

	function DownLowNul(DNLmin)
	 {
	   getDimensions(DNLwidth,DNLheight,DNLc,DNLd,DNLe);
	   for (i=0;i<DNLwidth;i++)
	    {
	     for (j=0;j<DNLheight;j++)
	      {
		if(getPixel(i,j)<=DNLmin)
		 setPixel(i,j,0);
	      }
		}
	  }

// ===============================
// ==== Smart Dialog Position ====

	function DialogPositionX (ImaLoc, xLocShift)
	  {	

		selectWindow(ImaLoc);
		getLocationAndSize(picture_x , picture_y , picture_width , picture_height);
			dialog_x=picture_x-xLocShift;
			   if (dialog_x<=0)  
		                   dialog_x=1;
	return dialog_x
	  }
	
	function DialogPositionY (ImaLoc)
	  {	

		selectWindow(ImaLoc);
		getLocationAndSize(picture_x , picture_y , picture_width , picture_height);
			dialog_y=picture_height / 3+picture_y ;
			  if (dialog_y>=screenHeight-20) 
			      dialog_y=1;
	return dialog_y
	  }

// ========================
// ==== Roue de couleur ===

function colorWheel (XcW)
	{
	    colorWiList=newArray("Blue","Red","Green","Cyan","Magenta","Yellow","Black");
	  	 while(XcW>6)
		XcW-=7;
	  return colorWiList [XcW]
	}

// ==============
// ==== date ====

     function horodate()
	{
	  getDateAndTime(year,month,dayOfWeek,dayOfMonth,hour,minute,second,msec);
	  return""+year+"."+IJ.pad(month+1,2)+"."+IJ.pad(dayOfMonth,2);
	}

// ==========================
// ==== Close All Windows ===

     function ScreenClean()
      {	
		while (nImages > 0) close();
		WinOp = getList("window.titles");
		WinOp = Array.deleteValue(WinOp, "Command Finder");
		for(i = 0; i < WinOp.length; i++)
		close(WinOp[i]);
       }
